@@ -4,7 +4,8 @@ from typing import Any, cast
 
 from langchain_core.language_models import BaseChatModel
 from langchain_core.tools import BaseTool
-from langgraph.graph.state import CompiledStateGraph, StateGraph
+from langgraph.checkpoint.base import BaseCheckpointSaver
+from langgraph.graph.state import CompiledStateGraph
 
 from src.agents import ContextSchemaType, StateSchemaType
 from src.agents.deep_agent import create_deep_agent
@@ -30,8 +31,9 @@ class AgentFactory:
         llm: BaseChatModel,
         prompt: str,
         state_schema: StateSchemaType,
-        internal_tools: list[BaseTool] | None = None,
         context_schema: ContextSchemaType | None = None,
+        checkpointer: BaseCheckpointSaver | None = None,
+        internal_tools: list[BaseTool] | None = None,
         subagents: list[SubAgent] | None = None,
     ) -> CompiledStateGraph:
 
@@ -43,6 +45,7 @@ class AgentFactory:
             prompt=prompt,
             state_schema=state_schema,
             context_schema=context_schema,
+            checkpointer=checkpointer,
             subagents=subagents,
         )
 
@@ -173,22 +176,24 @@ class GraphFactory:
         state_schema: StateSchemaType,
         context_schema: ContextSchemaType | None,
         mcp_config: MCPConfig,
+        checkpointer: BaseCheckpointSaver | None = None,
         llm_config: LLMConfig | None = None,
         template_context: dict[str, Any] | None = None,
-    ) -> StateGraph:
+    ) -> CompiledStateGraph:
         """Create a compiled graph with optional checkpointer support.
 
         Args:
             config: Agent configuration including checkpointer settings
             state_schema: State schema for the graph
+            context_schema: Optional context schema for the graph
             mcp_config: MCP configuration for tool loading
+            checkpointer: Optional checkpoint saver
             llm_config: Optional LLM configuration to override the one in config
             template_context: Optional template variables for prompt rendering
 
-        Yields:
-            StateGraph: The state graph
+        Returns:
+            CompiledStateGraph: The state graph
         """
-        builder = StateGraph(state_schema)
         mcp_client = await self.mcp_factory.create(mcp_config)
 
         all_impl_tools = self.tool_factory.get_impl_tools()
@@ -249,15 +254,9 @@ class GraphFactory:
             prompt=rendered_prompt,
             state_schema=state_schema,
             context_schema=context_schema,
+            checkpointer=checkpointer,
             subagents=resolved_subagents,
         )
-        builder.add_node(
-            config.name,
-            agent,
-        )
-
-        builder.set_entry_point(config.name)
-        builder.set_finish_point(config.name)
         # Store tools for cache access
-        builder._tools = tools + internal_tools  # type: ignore
-        return builder
+        agent._tools = tools + internal_tools  # type: ignore
+        return agent
