@@ -27,6 +27,11 @@ def add_reducer(left: int | None, right: int | None) -> int:
     return (left or 0) + (right or 0)
 
 
+def replace_reducer(left: int | None, right: int | None) -> int:
+    """Replace with new value, treating None as 0."""
+    return right if right is not None else (left or 0)
+
+
 def sum_reducer(left: float | None, right: float | None) -> float:
     """Sum two floats, treating None as 0.0."""
     return (left or 0.0) + (right or 0.0)
@@ -37,6 +42,6 @@ class AgentState(BaseAgentState):
 
     todos: list[Todo] | None
     files: Annotated[dict[str, str] | None, file_reducer]
-    current_input_tokens: Annotated[int | None, add_reducer]
+    current_input_tokens: Annotated[int | None, replace_reducer]
     current_output_tokens: Annotated[int | None, add_reducer]
     total_cost: Annotated[float | None, sum_reducer]
