@@ -3,8 +3,7 @@
 from collections.abc import AsyncIterator
 from contextlib import asynccontextmanager
 
-from langgraph.checkpoint.base import BaseCheckpointSaver
-
+from src.checkpointer.base import BaseCheckpointer
 from src.checkpointer.impl.memory import MemoryCheckpointer
 from src.checkpointer.impl.sqlite import AsyncSqliteCheckpointer
 from src.core.config import CheckpointerConfig, CheckpointerProvider
@@ -13,18 +12,22 @@ from src.core.config import CheckpointerConfig, CheckpointerProvider
 class CheckpointerFactory:
     """Factory for creating checkpointer instances."""
 
+    def __init__(self):
+        self._memory_instance: MemoryCheckpointer | None = None
+
     @asynccontextmanager
     async def create(
         self, config: CheckpointerConfig, database_url: str
-    ) -> AsyncIterator[BaseCheckpointSaver]:
+    ) -> AsyncIterator[BaseCheckpointer]:
         """Create checkpointer based on config type."""
         if config.type == CheckpointerProvider.SQLITE:
             async with AsyncSqliteCheckpointer.create(
                 connection_string=database_url,
-                max_connections=config.max_connections,
             ) as checkpointer:
                 yield checkpointer
         elif config.type == CheckpointerProvider.MEMORY:
-            yield MemoryCheckpointer()
+            if self._memory_instance is None:
+                self._memory_instance = MemoryCheckpointer()
+            yield self._memory_instance
         else:
             raise ValueError(f"Unknown checkpointer provider: {config.type}")
