@@ -1,6 +1,6 @@
 """Slash command completer."""
 
-from collections.abc import Iterable
+from collections.abc import AsyncGenerator, Iterable
 
 from prompt_toolkit.completion import CompleteEvent, Completion, WordCompleter
 from prompt_toolkit.document import Document
@@ -15,5 +15,12 @@ class SlashCommandCompleter(WordCompleter):
     def get_completions(
         self, document: Document, complete_event: CompleteEvent
     ) -> Iterable[Completion]:
-        """Get completions for slash commands."""
+        """Get completions for slash commands (sync)."""
         yield from super().get_completions(document, complete_event)
+
+    async def get_completions_async(
+        self, document: Document, complete_event: CompleteEvent
+    ) -> AsyncGenerator[Completion]:
+        """Get completions for slash commands (async)."""
+        for completion in self.get_completions(document, complete_event):
+            yield completion
