@@ -15,6 +15,11 @@ CONFIG_CHECKPOINTS_URL_FILE_NAME = Path(f"{CONFIG_DIR_NAME}/config.checkpoints.d
 CONFIG_HISTORY_FILE_NAME = Path(f"{CONFIG_DIR_NAME}/.history")
 CONFIG_MEMORY_FILE_NAME = Path(f"{CONFIG_DIR_NAME}/memory.md")
 
+CONFIG_LLMS_DIR = Path(f"{CONFIG_DIR_NAME}/llms")
+CONFIG_CHECKPOINTERS_DIR = Path(f"{CONFIG_DIR_NAME}/checkpointers")
+CONFIG_AGENTS_DIR = Path(f"{CONFIG_DIR_NAME}/agents")
+CONFIG_SUBAGENTS_DIR = Path(f"{CONFIG_DIR_NAME}/subagents")
+
 DEFAULT_CONFIG_DIR_NAME = "resources/configs/default"
 
 MAX_CONCURRENT_SUBAGENTS = 3
