@@ -1,13 +1,12 @@
 import shutil
-from typing import Annotated
 
+from langchain.tools import ToolRuntime, tool
 from langchain_core.messages import ToolMessage
-from langchain_core.runnables import RunnableConfig
-from langchain_core.tools import InjectedToolCallId, ToolException
+from langchain_core.tools import ToolException
 from pydantic import BaseModel, Field
 
+from src.agents.context import AgentContext
 from src.cli.theme import theme
-from src.tools.wrapper import approval_tool
 from src.utils.path import resolve_path
 from src.utils.render import format_diff_rich, generate_diff
 
@@ -23,10 +22,11 @@ class MoveOperation(BaseModel):
     """Represents a single file move operation."""
 
     source: str = Field(
-        ..., description="Source file path (relative to working directory)"
+        ..., description="Source file path (relative to working directory or absolute)"
     )
     destination: str = Field(
-        ..., description="Destination file path (relative to working directory)"
+        ...,
+        description="Destination file path (relative to working directory or absolute)",
     )
 
 
@@ -37,11 +37,10 @@ def _get_attr(obj: dict | BaseModel, attr: str, default: str = "") -> str:
     return getattr(obj, attr, default)
 
 
-def _render_diff_args(args: dict, config: RunnableConfig) -> str:
+def _render_diff_args(args: dict, config: dict) -> str:
     """Render arguments with colored diff preview."""
     file_path = args.get("file_path", "")
 
-    # Read full file content for accurate line numbers
     working_dir = config.get("configurable", {}).get("working_dir")
     full_content = None
     if working_dir and file_path:
@@ -52,10 +51,8 @@ def _render_diff_args(args: dict, config: RunnableConfig) -> str:
         except Exception:
             pass
 
-    # Check if this is multi-edit format or single edit
     edits = args.get("edits")
     if edits:
-        # Multi-edit format: generate diff for each section
         all_diff_sections = []
         for edit in edits:
             old_content = _get_attr(edit, "old_content")
@@ -65,7 +62,6 @@ def _render_diff_args(args: dict, config: RunnableConfig) -> str:
             )
             all_diff_sections.append(diff_lines)
 
-        # Combine all diff sections with separator
         combined_diff = []
         for i, diff_section in enumerate(all_diff_sections):
             if i > 0:
@@ -87,25 +83,23 @@ def _render_diff_args(args: dict, config: RunnableConfig) -> str:
     )
 
 
-@approval_tool(name_only=True)
+@tool
 async def read_file(
-    config: RunnableConfig,
-    tool_call_id: Annotated[str, InjectedToolCallId],
+    runtime: ToolRuntime[AgentContext],
     file_path: str,
-    start_line: int,
+    start_line: int = 0,
     limit: int = 500,
 ) -> ToolMessage:
     """
     Use this tool to read the content of a file with line-based pagination.
 
     Args:
-        file_path (str): Path to the file to read (relative to working directory)
-        start_line (int): Starting line number (0-based)
-        limit (int): Maximum number of lines to read (default: 500)
+        file_path: Path to the file to read (relative to working directory or absolute)
+        start_line: Starting line number (0-based) to read from (default: 0)
+        limit: Maximum number of lines to read (default: 500)
     """
-    working_dir = config.get("configurable", {}).get("working_dir")
-    if not working_dir:
-        raise ToolException("Working directory is not configured.")
+    context: AgentContext = runtime.context
+    working_dir = str(context.working_dir)
 
     path = resolve_path(working_dir, file_path)
 
@@ -114,57 +108,56 @@ async def read_file(
 
     total_lines = len(all_lines)
 
-    # Validate bounds
     start_idx = max(0, start_line)
     end_idx = min(total_lines, start_idx + limit)
 
-    # Get the requested lines
     selected_lines = all_lines[start_idx:end_idx]
 
-    # Add line numbers to content (0-based)
     numbered_content = "\n".join(
         f"{i + start_idx:4d} - {line.rstrip()}" for i, line in enumerate(selected_lines)
     )
 
-    # Generate short content summary
     actual_end = start_idx + len(selected_lines) - 1 if selected_lines else start_idx
     short_content = (
         f"Read {start_idx}-{actual_end} of {total_lines} lines from {path.name}"
     )
 
-    # Add summary at the end for LLM context
     lines_read = len(selected_lines)
     content_with_summary = f"{numbered_content}\n\n[{start_idx}-{actual_end}, {lines_read}/{total_lines} lines]"
 
     return ToolMessage(
         name=read_file.name,
         content=content_with_summary,
-        tool_call_id=tool_call_id,
+        tool_call_id=runtime.tool_call_id,
         short_content=short_content,
     )
 
 
-@approval_tool(name_only=True, render_args_fn=_render_diff_args)
+read_file.metadata = {
+    "approval_config": {
+        "name_only": True,
+    }
+}
+
+
+@tool
 async def write_file(
-    config: RunnableConfig,
-    tool_call_id: Annotated[str, InjectedToolCallId],
     file_path: str,
     content: str,
+    runtime: ToolRuntime[AgentContext],
 ) -> ToolMessage:
     """
     Use this tool to create a new file with content. Only for files that don't exist yet.
     If the file already exists, use edit_file instead.
 
     Args:
-        file_path (str): Path to the file to write (relative to working directory)
-        content (str): Content to write to the file
+        file_path: Path to the file to write (relative to working directory or absolute)
+        content: Content to write to the file
     """
-    working_dir = config.get("configurable", {}).get("working_dir")
-    if not working_dir:
-        raise ToolException("Working directory is not configured.")
+    context: AgentContext = runtime.context
+    working_dir = str(context.working_dir)
     path = resolve_path(working_dir, file_path)
 
-    # Check if file already exists
     if path.exists():
         raise ToolException(f"File already exists: {path}. Use edit_file instead.")
 
@@ -172,60 +165,59 @@ async def write_file(
     with open(path, "w", encoding="utf-8") as f:
         f.write(content)
 
-    # Generate diff for short_content (file is new, so old_content is empty)
     diff_lines = generate_diff("", content, context_lines=3)
     short_content = format_diff_rich(diff_lines)
 
     return ToolMessage(
         name=write_file.name,
         content=f"File written: {path}",
-        tool_call_id=tool_call_id,
+        tool_call_id=runtime.tool_call_id,
         short_content=short_content,
     )
 
 
-@approval_tool(name_only=True, render_args_fn=_render_diff_args)
+write_file.metadata = {
+    "approval_config": {
+        "name_only": True,
+        "render_args_fn": _render_diff_args,
+    }
+}
+
+
+@tool
 async def edit_file(
-    config: RunnableConfig,
-    tool_call_id: Annotated[str, InjectedToolCallId],
     file_path: str,
     edits: list[EditOperation],
+    runtime: ToolRuntime[AgentContext],
 ) -> ToolMessage:
     """
     Use this tool to edit a file by replacing old content with new content.
 
     Args:
-        file_path (str): Path to the file to edit (relative to working directory)
-        edits (list[EditOperation]): Edit operations to apply sequentially
+        file_path: Path to the file to edit (relative to working directory or absolute)
+        edits: Edit operations to apply sequentially
     """
-    working_dir = config.get("configurable", {}).get("working_dir")
-    if not working_dir:
-        raise ToolException("Working directory is not configured.")
+    context: AgentContext = runtime.context
+    working_dir = str(context.working_dir)
     path = resolve_path(working_dir, file_path)
 
-    # Check if file exists
     if not path.exists():
         raise ToolException(f"File does not exist: {path}")
 
-    # Read current content
     with open(path, encoding="utf-8") as f:
         current_content = f.read()
 
-    # Validate all old_content strings exist before making any changes
     for edit in edits:
         if edit.old_content not in current_content:
             raise ToolException(f"Old content not found in file: {path}")
 
-    # Apply all replacements sequentially
     updated_content = current_content
     for edit in edits:
         updated_content = updated_content.replace(edit.old_content, edit.new_content)
 
-    # Write updated content
     with open(path, "w", encoding="utf-8") as f:
         f.write(updated_content)
 
-    # Generate diff for each edit section separately
     all_diff_sections = []
     for edit in edits:
         diff_lines = generate_diff(
@@ -236,7 +228,6 @@ async def edit_file(
         )
         all_diff_sections.append(diff_lines)
 
-    # Combine all diff sections with separator
     combined_diff = []
     for i, diff_section in enumerate(all_diff_sections):
         if i > 0:
@@ -248,49 +239,60 @@ async def edit_file(
     return ToolMessage(
         name=edit_file.name,
         content=f"File edited: {path}",
-        tool_call_id=tool_call_id,
+        tool_call_id=runtime.tool_call_id,
         short_content=short_content,
     )
 
 
-@approval_tool(name_only=True)
+edit_file.metadata = {
+    "approval_config": {
+        "name_only": True,
+        "render_args_fn": _render_diff_args,
+    }
+}
+
+
+@tool
 async def create_dir(
-    config: RunnableConfig,
-    tool_call_id: Annotated[str, InjectedToolCallId],
     dir_path: str,
+    runtime: ToolRuntime[AgentContext],
 ) -> str:
     """
     Use this tool to create a directory recursively.
 
     Args:
-        dir_path (str): Path to the directory to create (relative to working directory)
+        dir_path: Path to the directory to create (relative to working directory or absolute)
     """
-    working_dir = config.get("configurable", {}).get("working_dir")
-    if not working_dir:
-        raise ToolException("Working directory is not configured.")
+    context: AgentContext = runtime.context
+    working_dir = str(context.working_dir)
     path = resolve_path(working_dir, dir_path)
 
     path.mkdir(parents=True, exist_ok=True)
     return f"Directory created: {path}"
 
 
-@approval_tool(name_only=True)
+create_dir.metadata = {
+    "approval_config": {
+        "name_only": True,
+    }
+}
+
+
+@tool
 async def move_file(
-    config: RunnableConfig,
-    tool_call_id: Annotated[str, InjectedToolCallId],
     source_path: str,
     destination_path: str,
+    runtime: ToolRuntime[AgentContext],
 ) -> str:
     """
     Use this tool to move a file from source to destination.
 
     Args:
-        source_path (str): Path to the source file (relative to working directory)
-        destination_path (str): Path to the destination (relative to working directory)
+        source_path: Path to the source file (relative to working directory or absolute)
+        destination_path: Path to the destination (relative to working directory or absolute)
     """
-    working_dir = config.get("configurable", {}).get("working_dir")
-    if not working_dir:
-        raise ToolException("Working directory is not configured.")
+    context: AgentContext = runtime.context
+    working_dir = str(context.working_dir)
     src = resolve_path(working_dir, source_path)
     dst = resolve_path(working_dir, destination_path)
 
@@ -299,21 +301,26 @@ async def move_file(
     return f"File moved: {src} -> {dst}"
 
 
-@approval_tool(name_only=True)
+move_file.metadata = {
+    "approval_config": {
+        "name_only": True,
+    }
+}
+
+
+@tool
 async def move_multiple_files(
-    config: RunnableConfig,
-    tool_call_id: Annotated[str, InjectedToolCallId],
     moves: list[MoveOperation],
+    runtime: ToolRuntime[AgentContext],
 ) -> str:
     """
     Use this tool to move multiple files in one operation.
 
     Args:
-        moves (list[MoveOperation]): List of move operations to apply
+        moves: List of move operations to apply
     """
-    working_dir = config.get("configurable", {}).get("working_dir")
-    if not working_dir:
-        raise ToolException("Working directory is not configured.")
+    context: AgentContext = runtime.context
+    working_dir = str(context.working_dir)
     results = []
     for move in moves:
         src = resolve_path(working_dir, move.source)
@@ -324,46 +331,56 @@ async def move_multiple_files(
     return f"Files moved: {', '.join(results)}"
 
 
-@approval_tool(name_only=True)
+move_multiple_files.metadata = {
+    "approval_config": {
+        "name_only": True,
+    }
+}
+
+
+@tool
 async def delete_file(
-    config: RunnableConfig,
-    tool_call_id: Annotated[str, InjectedToolCallId],
     file_path: str,
+    runtime: ToolRuntime[AgentContext],
 ) -> str:
     """
     Use this tool to delete a file.
 
     Args:
-        file_path (str): Path to the file to delete (relative to working directory)
+        file_path: Path to the file to delete (relative to working directory or absolute)
     """
-    working_dir = config.get("configurable", {}).get("working_dir")
-    if not working_dir:
-        raise ToolException("Working directory is not configured.")
+    context: AgentContext = runtime.context
+    working_dir = str(context.working_dir)
     path = resolve_path(working_dir, file_path)
 
     path.unlink()
     return f"File deleted: {path}"
 
 
-@approval_tool(name_only=True, render_args_fn=_render_diff_args)
+delete_file.metadata = {
+    "approval_config": {
+        "name_only": True,
+    }
+}
+
+
+@tool
 async def insert_at_line(
-    config: RunnableConfig,
-    tool_call_id: Annotated[str, InjectedToolCallId],
     file_path: str,
     line_number: int,
     content: str,
+    runtime: ToolRuntime[AgentContext],
 ) -> ToolMessage:
     """
     Use this tool to insert content at a specific line number.
 
     Args:
-        file_path (str): Path to the file (relative to working directory)
-        line_number (int): Line number to insert at (1-based, content inserted before this line)
-        content (str): Content to insert
+        file_path: Path to the file (relative to working directory or absolute)
+        line_number: Line number to insert at (1-based, content inserted before this line)
+        content: Content to insert
     """
-    working_dir = config.get("configurable", {}).get("working_dir")
-    if not working_dir:
-        raise ToolException("Working directory is not configured.")
+    context: AgentContext = runtime.context
+    working_dir = str(context.working_dir)
     path = resolve_path(working_dir, file_path)
 
     if not path.exists():
@@ -372,36 +389,29 @@ async def insert_at_line(
     if line_number < 1:
         raise ToolException(f"Line number must be >= 1: {line_number}")
 
-    # Read current content
     with open(path, encoding="utf-8") as f:
         old_content = f.read()
         lines = old_content.splitlines(keepends=True)
 
     total_lines = len(lines)
 
-    # Allow inserting at end of file (line_number = total_lines + 1)
     if line_number > total_lines + 1:
         raise ToolException(
             f"Line number {line_number} exceeds file length ({total_lines} lines)"
         )
 
-    # Insert content at specified line (convert to 0-based index)
     insert_index = line_number - 1
 
-    # Ensure content ends with newline if inserting in middle of file
     if not content.endswith("\n") and insert_index < total_lines:
         content = content + "\n"
 
-    # Split content into lines and insert
     new_lines = content.splitlines(keepends=True)
     lines[insert_index:insert_index] = new_lines
 
-    # Write updated content
     new_content = "".join(lines)
     with open(path, "w", encoding="utf-8") as f:
         f.write(new_content)
 
-    # Generate diff for short_content (use Rich formatting for colored output)
     diff_lines = generate_diff(
         old_content, new_content, context_lines=3, full_content=old_content
     )
@@ -411,40 +421,49 @@ async def insert_at_line(
     return ToolMessage(
         name=insert_at_line.name,
         content=f"Inserted {inserted_line_count} line(s) at line {line_number} in {path}",
-        tool_call_id=tool_call_id,
+        tool_call_id=runtime.tool_call_id,
         short_content=short_content,
     )
 
 
-@approval_tool()
+insert_at_line.metadata = {
+    "approval_config": {
+        "name_only": True,
+        "render_args_fn": _render_diff_args,
+    }
+}
+
+
+@tool
 async def delete_dir(
-    config: RunnableConfig,
-    tool_call_id: Annotated[str, InjectedToolCallId],
     dir_path: str,
+    runtime: ToolRuntime[AgentContext],
 ) -> str:
     """
     Use this tool to delete a directory recursively.
 
     Args:
-        dir_path (str): Path to the directory to delete (relative to working directory)
+        dir_path: Path to the directory to delete (relative to working directory or absolute)
     """
-    working_dir = config.get("configurable", {}).get("working_dir")
-    if not working_dir:
-        raise ToolException("Working directory is not configured.")
+    context: AgentContext = runtime.context
+    working_dir = str(context.working_dir)
     path = resolve_path(working_dir, dir_path)
 
     shutil.rmtree(path)
     return f"Directory deleted: {path}"
 
 
+delete_dir.metadata = {"approval_config": {}}
+
+
 FILE_SYSTEM_TOOLS = [
     read_file,
     write_file,
     edit_file,
-    insert_at_line,
     create_dir,
     move_file,
     move_multiple_files,
     delete_file,
+    insert_at_line,
     delete_dir,
 ]
