@@ -17,7 +17,7 @@ class TestCheckpointerFactory:
     @pytest.mark.asyncio
     async def test_create_sqlite_checkpointer(self):
         factory = CheckpointerFactory()
-        config = CheckpointerConfig(type=CheckpointerProvider.SQLITE, max_connections=5)
+        config = CheckpointerConfig(type=CheckpointerProvider.SQLITE)
 
         async with factory.create(config, "file::memory:?cache=shared") as checkpointer:
             assert checkpointer is not None
